/*
******************************************************************************
**  CarMaker - Version 7.0
**  Vehicle Dynamics Simulation Toolkit
**
**  Copyright (C)   IPG Automotive GmbH
**                  Bannwaldallee 60             Phone  +49.721.98520.0
**                  76185 Karlsruhe              Fax    +49.721.98520.99
**                  Germany                      WWW    www.ipg-automotive.com
******************************************************************************
**
** Simple tire Model
**
** Add the declaration of the register function to one of your header files,
** for example to User.h and call it in User_Register()
**
**    TireCPMod_Register_Kick_Plate ();
**
******************************************************************************
*/

#include <stdlib.h>
#include <string.h>
#include <math.h>

#include "CarMaker.h"
#include "Car/Vehicle_Car.h"
#include "MyModels.h"
#include "User.h"

static const char ThisModelClass[] = "TireCPMod";
static const char ThisModelKind_KP[]  = "TireCPMod_KickPlate";
static const int  ThisVersionId_KP    = 1;


struct tMyModel_KP {
    /* Model specific Parameters */
    int TireNo;
    double LatVel; /* lateral velocity at contact point in Fr0 Frame*/
};


static void
MyModel_DeclQuants_dyn_KP (struct tMyModel_KP *mp, int park)
{

    tDDefault *df;
    static struct tMyModel_KP MyModel_Dummy_KP = {0};

    if (park) {
	   	    mp = &MyModel_Dummy_KP;
}
    /* Define here dict entries for dynamically allocated variables. */
    df = DDefaultCreate(NULL);
    DDefPrefix(df, "TireCPMod.%s.", Tire_TireNo_Str(mp->TireNo));
    DDefDouble4 (df, "LatVel",	"m/s",	&mp->LatVel, DVA_VC);
    DDefaultDelete (df);
}


static void
MyModel_DeclQuants_KP (void *MP)
{

    struct tMyModel_KP *mp = (struct tMyModel_KP *)MP;

    if (mp == NULL) {
	/* Define here dict entries for non-dynamically allocated (static) variables. */

} else {
	MyModel_DeclQuants_dyn_KP (mp, 0);
}
}


static void *
    MyModel_New_KP (struct tInfos *inf, int TireNo, int Side, int Twin)
{

    struct tMyModel_KP *mp = (struct tMyModel_KP*)calloc(1,sizeof(*mp));
    mp->TireNo = TireNo;

    return mp;
}
static int
MyModel_Calc_KP (void *MP, tTireCPModIF *IF, double dt)
{

    struct tMyModel_KP *mp = (struct tMyModel_KP *)MP;
    if (IF->CP_0[0] > 106 && IF->CP_0[0] < 108) {
        mp->LatVel = KickPlate_LatVel;
	IF->CP_Vel_0[1] += mp->LatVel;
    }
    return 0;
}


static void
MyModel_Delete_KP (void *MP)
{
    struct tMyModel_KP *mp = (struct tMyModel_KP *)MP;
    /* Park the dict entries for dynamically allocated variables before deleting */
    MyModel_DeclQuants_dyn_KP (mp, 1);
    free (mp);
}


int
TireCPMod_Register_Kick_Plate (void)
{
    tModelClassDescr m;

    memset(&m, 0, sizeof(m));
    m.TireCPMod.VersionId =	ThisVersionId_KP;
    m.TireCPMod.New =		MyModel_New_KP;
    m.TireCPMod.Calc =		MyModel_Calc_KP;
    m.TireCPMod.Delete =	MyModel_Delete_KP;
    m.TireCPMod.DeclQuants =	MyModel_DeclQuants_KP;
    /* Should only be used if the model doesn't read params from extra files */
    m.TireCPMod.ParamsChanged = ParamsChanged_IgnoreCheck;

    return Model_Register(ModelClass_TireCPMod, ThisModelKind_KP, &m);
}


